from flask_shop.category import category_api, category_bp, attribute_api, attribute_bp
from flask import request
from flask_shop import models, db
from flask_shop.utils.message import state_message
from flask_restful import Resource


# 商品分类接口
class Category(Resource):
    # 获取商品分类接口
    def get(self):
        try:
            cid = request.args.get('cid')
            c = models.Category.query.get(cid)
            if c:
                return state_message(200, data=c.to_dict(), message="获取商品分类成功！")
            return state_message(10022)
        except Exception as e:
            print(e)
            return state_message(20000)

    # 增加商品分类接口
    def post(self):
        try:
            name = request.form.get('name') if request.form.get('name') else ''
            level = int(request.form.get(
                'level')) if request.form.get('level') else None
            pid = int(request.form.get('pid')
                      ) if request.form.get('pid') else None

            if all([name, level]):
                c = models.Category(name=name, level=level, pid=pid)
                db.session.add(c)
                db.session.commit()
                return state_message(200, message='添加商品分类成功！')
            return state_message(10000)
        except Exception as e:
            print(e)
            return state_message(20000)

    # 修改商品分类接口
    def put(self):
        try:
            cid = request.form.get('cid')
            name = request.form.get('name')
            c = models.Category.query.get(cid)

            if c:
                c.name = name
                db.session.commit()
                return state_message(200, message="更新商品分类成功！")
            return state_message(10022)
        except Exception as e:
            print(e)
            return state_message(20000)

    # 删除商品分类接口
    def delete(self):
        try:
            cid = request.form.get('cid')
            c = models.Category.query.get(cid)
            if c:
                db.session.delete(c)
                db.session.commit()
                return state_message(200, message="删除商品分类成功！！")
            return state_message(10022)
        except Exception as e:
            print(e)
            return state_message(20000)


# 添加类视图和url
category_api.add_resource(Category, '/category')


# 获取商品分类列表树
@category_bp.route('/category_list')
def get_category_list():
    # 级别
    level = int(request.args.get('level')) if request.args.get(
        'level') and int(request.args.get('level')) < 3 else 0
    # 当前页码
    pnum = int(request.args.get('pnum')) if request.args.get('pnum') else 0
    # 每页显示条数
    psize = int(request.args.get('psize')) if request.args.get('psize') else 0

    # 储存返回结果
    cate_list = []
    # 基础查询
    base_query = models.Category.query.filter(models.Category.level == 1)
    if all([psize, pnum]):  # 如果有数据传过来说明要分页
        # 将获取到的分页数据，进行分页
        categories = base_query.paginate(pnum, psize)
        if level:   # 有等级
            cate_list = get_tree(categories.items, level, True)
        else:
            cate_list = get_tree(categories.items, level, False)
        data = {
            'pnum': pnum,
            'psize': psize,
            # total:总页数
            'total': categories.total,
            'data': cate_list
        }
        return state_message(200, data=data, message="获取商品分类列表成功！")
    else:
        # 不实现分页效果
        # 获取level=1的所有数据
        categorys = base_query.all()
        if level:   # 不分页控制深度
            cate_list = get_tree(categorys, level, True)
        else:   # 不分页不要深度
            # for c in categorys:
            #     # 一级目录
            #     first_dict = c.to_dict()
            #     # 在一级目录为二级目录创建容器
            #     first_dict['children'] = []

            #     # 遍历二级目录
            #     for sc in c.children:
            #         sc_dict = sc.to_dict()
            #         # 在二级目录为三级目录创建容器
            #         sc_dict['children'] = []

            #         # 遍历三级目录
            #         for tc in sc.children:
            #             # 将遍历到的三级目录，加到在二级目录为三级目录创建容器里面
            #             sc_dict['children'].append(tc.to_dict())
            #         # 将遍历到的二级目录，加到在一级目录为二级目录创建容器里面
            #         first_dict['children'].append(sc_dict)
            #     # 最后将一级目录加到 cate_list = [] 中
            #     cate_list.append(first_dict)
            cate_list = get_tree(categorys, level, False)
        return state_message(200, data={"data": cate_list}, message="获取商品分类列表成功！")


# 使用递归获取目录
"""
info_list:要遍历的列表
level: 层数
flag: 控制level是否有效
"""


def get_tree(info_list, level, flag):
    info_dict = []
    # 判断info_list是否有值
    if info_list:
        for i in info_list:
            i_dict = i.to_dict()
            if flag:
                if i.level < level:
                    # 调用自己传入当前节点
                    i_dict['children'] = get_tree(i.children, level, flag)
            else:
                if i.level < 3:    # 说明拥有子节点
                    i_dict['children'] = get_tree(i.children, level, flag)
            info_dict.append(i_dict)
    return info_dict


# attribute属性接口
class Attribute(Resource):
    # 分类属性参数增加接口
    def post(self):
        try:
            # 属性名称
            name = request.form.get('name')
            # 属性值
            val = request.form.get('val')
            # 枚举类型 'static', 'dynamic'
            _type = request.form.get('_type')
            cid = request.form.get('cid')

            if all([name, cid, _type]):
                if val:
                    attr = models.Attribute(name=name, val=val, _type=_type, cid=int(cid))
                else:
                    attr = models.Attribute(name=name, cid=int(cid), _type=_type)
                db.session.add(attr)
                db.session.commit()
                return state_message(200)
            else:
                return state_message(10000)
        except Exception as e:
            print(e)
            return state_message(20000)

    # ID获取分类参数
    def get(self):
        id = request.args.get('id')
        attr = models.Attribute.query.get(id)
        if attr:
            return state_message(200, data=attr.to_dict(), message="获取分类参数成功！")
        
        else:
            return state_message(10022)

    # 修改分类参数
    def put(self):
        try:
            id = request.form.get('id')
            name = request.form.get('name')
            val = request.form.get('val')
            _type = request.form.get('_type')
            cid = int(request.form.get('cid')) if request.form.get('cid') else None

            if all([id, name, cid]):
                attr = models.Attribute.query.get(id)
                if attr:
                    attr.name = name
                    attr.val = val
                    attr.cid = cid
                    db.session.commit()
                    return state_message(200, message="更新数据成功！")
                else:
                    return state_message(10022)
            else:
                return state_message(10000)
        except Exception as e:
            print(e)
            return state_message(20000)

    # 分类参数删除接口实现  
    def delete(self):
        try:
            id = request.form.get('id')

            attr = models.Attribute.query.get(id)
            if attr:
                db.session.delete(attr)
                db.session.commit()
                return state_message(200, data=attr.to_dict(), message="删除分类数据成功！")
            else:
                return state_message(10022)
        except Exception as e:
            print(e)
            return state_message(20000)

# 添加类视图和url
attribute_api.add_resource(Attribute, '/attribute')


# 分类参数获取列表
@attribute_bp.route('/attr_list')
def get_attr_list():
    try:
        # 要那个分类属性的数据
        cid = request.args.get('cid')
        # 要什么类型的数据
        _type = request.args.get('_type')

        if all([cid, _type]):
            # 通过少的一方查找多的一方
            cate = models.Category.query.get(cid)
            attr_list = []
            if cate:
                if _type == 'static':
                    # 静态数据
                    attr_list = [a.to_dict() for a in cate.attributes if a._type == 'static']
                else:
                    # 动态数据
                    attr_list = [a.to_dict() for a in cate.attributes if a._type == 'dynamic']
                return state_message(200, data=attr_list, message="获取分类属性列表成功！")
            else:
                return state_message(10022)
        else:
            return state_message(10000)
    except Exception as e:
        print(e)
        return state_message(20000)