from flask_shop.goods import goods_api, goods_bp
from flask_shop import models, db
from flask import request, current_app
from flask_restful import Resource
from flask_shop.utils.message import state_message
import hashlib
from time import time

# 创建路由


@goods_bp.route('/goods_list')
def get_goods_list():
    name = request.args.get('name')
    if name:
        goods = models.Goods.query.filter(
            models.Goods.name.like(f"%{name}%")).all()
    else:
        goods = models.Goods.query.all()
    goods_list = [goods_obj.to_dict() for goods_obj in goods]
    return state_message(200, data=goods_list, message="获取商品列表成功！")


class Goods(Resource):
    def post(self):
        try:
            attr_dynamic = request.form.get('attr_dynamic')
            attr_static = request.form.get('attr_static')
            pics = request.form.get('pics')

            cid_one = request.form.get('cid_one')
            cid_three = request.form.get('cid_three')
            cid_two = request.form.get('cid_two')
            introduce = request.form.get('introduce')
            name = request.form.get('name')
            number = request.form.get('number')
            price = request.form.get('price')
            weight = request.form.get('weight')

            # 创建商品模型对象
            goods = models.Goods(name=name, number=number, price=price,
                        weight=weight, introduce=introduce, cid_one=int(cid_one),
                        cid_two=int(cid_two), cid_three=int(cid_three))
            db.session.add(goods)
            db.session.commit()
            # 图片路径存储
            for p in eval(pics):
                tp = models.Picture(gid=goods.id, path=p)
                db.session.add(tp)
            # 静态参数存储
            for s in eval(attr_static):
                temp_s = models.GoodsAttr(gid=goods.id, aid=s.get(
                    'id'), val=s.get('val'), _type='static')
                db.session.add(temp_s)

            # 动态参数存储
            for d in eval(attr_dynamic):
                temp_d = models.GoodsAttr(gid=goods.id, aid=d.get(
                    'id'), val=d.get('val'), _type='dynamic')
                db.session.add(temp_d)
            db.session.commit()
            return state_message(200, message="增加商品成功！")
        except Exception as e:
            print(e)
            return state_message(20000)


    # 删除
    def delete(self):
        try:
            id = request.json.get('id')
            goods = models.Goods.query.get(id)
            if goods:
                db.session.delete(goods)
                db.session.commit()
                return state_message(200, message="删除商品成功！")
        except Exception as e:
            print(e)
            return state_message(10022)


goods_api.add_resource(Goods, '/goods_class')


# 上传图片接口
@goods_bp.route('/upload_img', methods=['POST'])
def upload_img():
    # 接收上传上来的文件参数
    img_file = request.files.get('file')
    if not img_file:
        return state_message(10023)

    if allowed_img(img_file.filename):
        # 保存文件的位置
        floder = current_app.config.get('SEVER_IMG_UPLOADS')
        # 获取文件后缀名
        end_prefix = img_file.filename.rsplit('.', 1)[1]
        # 获取文件名
        file_name = md5_file()
        # 保存文件
        img_file.save(f'{floder}/{file_name}.{end_prefix}')

        # 返回给前端保存的路径信息
        data = {
            'path': f'/static/img/{file_name}.{end_prefix}',
            'url': f'http://127.0.0.1:5000/static/img/{file_name}.{end_prefix}'
        }
        return state_message(200, data=data, message="图片上传成功！")
    else:
        return state_message(10024)


# 检查上传文件是否合法
def allowed_img(file_name):
    """
    '.' in file_name : 判断文件名是否有后缀
    file_name.splist('.', 1)[1] : 将文件名以.进行分割，并且分割一次,取第二个的值
    """
    return '.' in file_name and file_name.rsplit('.', 1)[1] in current_app.config['ALLOWED_IMG']


# 随机生成图片文件名
def md5_file():
    # 创建md5对象
    md5_obj = hashlib.md5()
    # 更新要加密时间戳数据
    md5_obj.update(str(time()).encode())
    # 加密出来的数据
    print(time())
    filename = md5_obj.hexdigest()
    return filename
