from flask_shop.role import role_bp, role_api
from flask_shop import models, db
from flask import request
from flask_restful import Resource
from flask_shop.utils.message import state_message


class Role(Resource):
    # 获取角色列表接口
    def get(self):
        role_list = []
        try:
            roles = models.Role.query.all()
            role_list = [r.to_dict() for r in roles]
            return state_message(200, data=role_list, message="获取角色列表成功！")
        except Exception as e:
            return state_message(20000)

    # 增加角色列表接口
    def post(self):
        name = request.form.get('name')
        desc = request.form.get('desc')
        try:
            if name:
                role = models.Role(name=name, desc=desc)
                db.session.add(role)
                db.session.commit()
                return state_message(200, message="增加角色成功！")
        except Exception as e:
            print(e)
            return state_message(20000)
    # 删除角色的接口
    def delete(self):
        try:
            id = int(request.form.get('id'))
            role = models.Role.query.get(id)
            if role:
                db.session.delete(role)
                db.session.commit()
                return state_message(200, message="角色删除成功！")
            return state_message(10000)
        except Exception as e:
            print(e)
            return state_message(20000)
    
    # 修改角色接口
    def put(self):
        try:
            id = int(request.form.get('id'))
            name = request.form.get('name') if request.form.get('name') else ''
            desc = request.form.get('desc') if request.form.get('desc') else ''
            if name:
                role = models.Role.query.get(id)
                role.name = name
                role.desc = desc
                db.session.commit()
                return state_message(200, message="修改角色信息成功！！")
            return state_message(10000)
        except Exception as e:
            print(e)
            return state_message(20000)

        

# 添加类视图与url
role_api.add_resource(Role, '/role')

# 删除权限接口
@role_bp.route('/del_menu/<int:rid>/<int:mid>')
def del_menu(rid, mid):
    try:
        r = models.Role.query.get(rid)
        m = models.Menu.query.get(mid)
        if all([r,m]):
            if m in r.menus:
                r.menus.remove(m)
                if m.level == 1:
                    for temp_m in m.children:   # 获取当前根节点的所有子节点
                        if temp_m in r.menus:   # 判断当前子节点是否在当前权限
                            r.menus.remove(temp_m)
                db.session.commit()
                # 返回data 当前行的权限
                return state_message(200, data=r.get_menu_dict(), message='权限删除成功！')
            return state_message(10021)
    except Exception as e:
        return state_message(20000)

# 分配权限接口
@role_bp.route('/set_menu/<int:rid>', methods=['POST'])
def set_menu(rid):
    try:
        role = models.Role.query.get(rid)
        mids = request.form.get('mids')
        if role:
            # 将原来拥有的权限全部清空，以免重新赋值时重复
            role.menus = []
            # 将前端传过来的mid用,隔开
            for m in mids.split(','):
                # 判断遍历出来的mid是否为null
                if m:
                    # 不为空去数据库查找
                    temp_menu = models.Menu.query.get(int(m))
                    # 判断是否在数据里找到mid的字段
                    if temp_menu:
                        role.menus.append(temp_menu)
            db.session.commit()
            return state_message(200, message='分配权限成功！')
    except Exception as e:
        print(e)
        return state_message(20000)
